; ******************************************************************************************************************
; **  PROGRAM MAKED BY FADISHOP  	Card: FADICLOCK	http://www.fadishop.eu	FADITECO, S.L.U. 	Lleonard Garcia	**
; **  This program reads the temperature every x seconds, averaged every minute and stored in the SRAM            **
; ** 	Activated alarm 1 (every second) and Alarm 2 (every minute) connectadas to interrupt INT2 (B.2)         	**
; ** 	SLAVE ADDRESS IS STILL DS3232 I2C $ D0.    					      					**
; **  INTERRUPT ROUTING / RST JP6 (Blue jumper)	(I0=1, I1=0, I2=1, portC.7=0;  Rpullup=1)		             	** 
; ******************************************************************************************************************
; SETTINGS
#picaxe 28x2		
let dirsB=%10000000		; 1=output 0=input
let dirsC=%00000000		; 1=output 0=input
; C.3 I2C_SCL
; C.4 I2C_SDA
;adcsetup = %zy0000000000xxxx ; SETTING ANALOG(z=0-GND, y=0-Vcc   xxxx inputs adc 0000 0001 0011 0111 1111)
;setfreq em16			; Oscillator / external oscillator to 16Mhz.
device_DS3232SN:
symbol adress_slave_A =$D0	; Address 0xD0 DS3232 I2C only.
symbol @now=$00			; Internal adress register of seconds, minutes and hours.
symbol @today=$03			; Internal adress register of day, month and year-century.
symbol @alm1=$07			; Internal adress register alarm_1 (sec, min, hour and day / day).
symbol @alm2=$0B			; Internal adress register alarm2 (min, hour and day / day).
symbol @control=$0E		; (7)/EOSC (6)BBSQW (5)CONV (4-3)RS2-1 (2)INTCN (1)A2IE (0)A1IE
symbol @status=$0F		; (7)OSF (6)BB32KHZ (5-4)CRATE (3)EN32KHZ (2)BSY (1)A2F (0)A1F
symbol @offset=$10		; temperature offset. (adds / subtracts capacitance)
symbol @temp=$11			; Internal word temperature address($11-MSB_temperature and $12-LSB_temperature) .
symbol @test=$13			; Reserved for test.
symbol @sram=$14			; Internal address of the start of the block of 236 bytes SRAM.
symbol control=b16		; Register/shadow control DS3232.
symbol status=b18			; Register/shadow control/status DS3232.
; Program Definitions and Initialization
symbol seconds=b0
symbol minutes=b1
symbol hour=b2			
symbol day=b3
symbol date=b4
symbol mounth=b5
symbol year=b6
symbol T_msb=b7
symbol T_lsb=b8

symbol acumMsb=w19
symbol acumLsb=w20
symbol medMinMsb=w21
symbol medMinLsb=w22
symbol N_sample=b48
symbol @SramMin=$20					; address of the SRAM that stores the average

;INICIALITZACIONES:
let control =%00011100					; match default initialization.
let status =%11001000					; match default initialization.
call set_date_time					; change the date-time DS3232 RTC.
call set_alarm1						; sets alarma_1 in the DS3232 RTC .
call set_alarm2						; sets alarma_2 in the DS3232 RTC.
interrupt_settings:
hint2flag = 0						; clear the flag INT_2.
symbol hint_setting = %00000100 			; active interrupts INT2.
hintsetup hint_setting					; transfers such activation.
setintflags %00000100,%00000100 			; INT2 interrupts are enabled per falling edge.

main:
call	temperature_request
i2cslave adress_slave_A, i2cslow, i2cbyte		; I2C address of DS3232 (0xD0-0xD1).
readi2c @now,(seconds,minutes,hour,day)		; reading time 
;readi2c @today,(date,mounth,year)			; reading date.
readi2c @temp,(T_msb,T_lsb)				; temperature reading.
;readi2c @alm1,(b10,b11,b12,b13)			; alarm_1 reading.
;readi2c @control,(b17,b19)				; reading register.
;readi2c INICI,(b0,b1)
debug
PAUSE 2000
goto main


; INTERRUPT SEQUENCE
interrupt:
readi2c @status,(status)				; It reads the control and status registers.
let b29=status and %00000001
if b29!=0 then call alarm1				; If A1F = 1 runs Alarm1.
let b29=status and %00000010
if b29!=0 then call alarm2				; Yes A2F = 1 runs Alarm2.
hint2flag = 0						; Re-enable interrupts INT2.
hintsetup hint_setting					; Re-enable interrupts INT2.
setintflags %00000100,%00000100 			; Interrupts are enabled and INT2 INT1 by falling edge.
return							; After return is when you actually enable the interrupt.

alarm1:
let status=status and %11111110			; Clears the flag on DS3232 alarma_1.
writei2c @status,(status)				; Enable and delete flag A1F.
call	add_sample
return


alarm2:
let status=status and %11111101			; Clears the flag on DS3232 alarma_2.
writei2c @status,(status)				; Enable and delete flag A2F.
call seconds_average
let acumMsb=0
let acumLsb=0
let N_sample=0

let b50=@SramMin
let b51=bcdtobin minutes
let b51=b51*2
let b50=b50 + b51 
writei2c b50,(medMinMsb,medMinLsb)			
readi2c @SramMin,(b52,b53,b54,b55)
return


set_date_time:
symbol xseg=b30
symbol xminu=b31
symbol xhour=b32
symbol xsemdate=b33
symbol xdate=b34
symbol xmounth=b35
symbol xyear=b36
let xseg=$25						; @_internal DS3232: $00
let xminu=$00						; @_internal DS3232: $01
let xhour=$13						; @_internal DS3232: $02
let xsemdate=$00						; @_internal DS3232: $03
let xdate=$03						; @_internal DS3232: $04
let xmounth=$03						; @_internal DS3232: $05
let xyear=$12						; @_internal DS3232: $06
low xhour,6							; LOW: 24H          HIGH: AM/PM
i2cslave adress_slave_A, i2cslow, i2cbyte		; Direccin I2C del DS3232 (0XD0-0XD1).
writei2c @now,(xseg,xminu,xhour,xsemdate,xdate,xmounth,xyear); I2C address of DS3232 (0xD0-0xD1)
return


set_alarm1:
symbol a1_seg=b30
symbol a1_min=b31
symbol a1_hor=b32						
symbol a1_day=b33
symbol a1_mode=b34
let a1_seg=$00
let a1_min=$01
let a1_hor=$13						; Time Format->bit(6)=0:24h, bit(6)=1:AM/PM 12h.
let a1_day=$03						; Date Format-> bit(6)=0:date del mounth, bit(6)=1:date de la semana.
let a1_mode=4
low a1_hor,6						; LOW: 24H          	HIGH: AM/PM
low a1_day,6						; LOW: date  		HIGH: day of week.						;
select case a1_mode
case 0 
clearbit a1_seg,7  clearbit a1_min,7  clearbit a1_hor,7   clearbit a1_day,7 	;0: exactly (date, hours, minutes, second).
case 1 
clearbit a1_seg,7  clearbit a1_min,7  clearbit a1_hor,7   setbit a1_day,7	;1: exactly (hours, minutes, seconds).
case 2 
clearbit a1_seg,7  clearbit a1_min,7  setbit a1_hor,7  setbit a1_day,7		;2: exactly (minutes, seconds).
case 3 
clearbit a1_seg,7  setbit a1_min,7 setbit a1_hor,7  setbit a1_day,7		;3: exactly (seconds).
case 4 
setbit a1_seg,7 setbit a1_min,7 setbit a1_hor,7  setbit a1_day,7			;4: periodic generator 1 second.
end select

i2cslave adress_slave_A, i2cslow, i2cbyte		; I2C address of DS3232 (0xD0-0xD1).
writei2c @alm1,(a1_seg,a1_min,a1_hor,a1_day)	; Send configuration of alarm_1.
let control=control or %00000001			; Enabled interrupt of alarm_1 A1E.
let control=control or %00000100			; Enable interrupt alarm_1 in DS3232 INTCN.
let status=status and %11111110			; Clears flag A1F on DS3232 alarm_1.
writei2c @control,(control,status)			; Send status.
return

set_alarm2:
symbol a2_min=b31
symbol a2_hor=b32						
symbol a2_day=b33
symbol a2_modo=b34
let a2_min=$01
let a2_hor=$13						; Time format->bit(6)=0:24h, 		bit(6)=1:AM/PM 12h.
let a2_day=$03						; Date format-> bit(6)=0:date, 	bit(6)=1:day of week.
let a2_modo=4
low a2_hor,6						; LOW: 24H          HIGH: AM/PM
low a2_day,6						; LOW: date		  HIGH: day of week.						;
select case a2_modo
case 0 
clearbit a2_min,7  clearbit a2_hor,7   clearbit a2_day,7 	;0: exactly (date, hours, minutes).
case 1 
clearbit a2_min,7  clearbit a2_hor,7   high a2_day,7		;1: exactly (hours, minutes).
case 2 
clearbit a2_min,7  setbit a2_hor,7  setbit a2_day,7		;2: exactly (minutes).
case 3 
setbit a2_min,7 setbit a2_hor,7  setbit a2_day,7		;3: periodic generator 1 second.
case 4 
setbit a2_min,7 setbit a2_hor,7  setbit a2_day,7		;4: periodic generator 1 second.
end select


i2cslave adress_slave_A, i2cslow, i2cbyte		; I2C address of DS3232 (0xD0-0xD1).
writei2c @alm2,(a2_min,a2_hor,a2_day)		; Send configuration of alarm_2.
let control=control or %00000010			; Enabled interrupt of alarm_2 A2E.
let control=control or %00000100			; Enable interrupt alarm_2 in DS3232 INTCN.
let status=status and %11111101			; Clears flag A2F on DS3232 alarm_2.
writei2c @control,(control,status)			; Send status.
return


add_sample:
acumMsb=T_msb+acumMsb
acumLsb=T_lsb+acumLsb
inc N_sample
return

seconds_average:
medMinMsb=acumMsb/N_sample
medMinLsb=acumLsb/N_sample
return

temperature_request:
i2cslave adress_slave_A, i2cslow, i2cbyte		; I2C address of DS3232 (0xD0-0xD1).
readi2c @status,(b10)					; Read status byte of DS3232
let b10=b10 and %00000100				; Is performing a conversion (BSY=1)?
if b10<>0 then return					; (YES=1) Do nothing, come back.
          else let control=control or %00100000	; (NO=0) Force a new temperature conversion.
			i2cslave adress_slave_A, i2cslow, i2cbyte		;I2C address of DS3232 (0xD0-0xD1).
		   	writei2c @control,(control)	
endif
return
